from _Framework.ButtonElement import ButtonElement

from fingerboardcodes import *
from livezonelayout import channelAndNoteForLed

MIDI_NOTE_ON_STATUS = 144
MIDI_NOTE_OFF_STATUS = 128

class FingerboardButton(ButtonElement):
	"""Represents a ButtonElement adapted for the interface to the Ztar fingerboard."""
	def __init__(self, isMomentary, messageType, led):
		self.ledNumber = led
		self.channelOut, self.note = channelAndNoteForLed(led)
		ButtonElement.__init__(self, isMomentary, messageType, self.channelOut, self.note)
		self.lastSentFunction = -1
                self.lastSentValue = -1
	
	def disconnect(self):
		ButtonElement.disconnect(self)
	
	def _do_send_value(self, value, channel=None):
                channel = channel or self.channelOut
		statusByte = MIDI_NOTE_ON_STATUS if value != 0 else MIDI_NOTE_OFF_STATUS
		statusByte += channel
		if self.send_midi((statusByte, self.note, value)):
			self.lastSentValue = value
			if self._report_output:
				is_input = True
				self._report_value(value, (not is_input))
	
	def send_function(self, functionCode, force_send=False):
		"""Uses the designated function channel (in fingerboardcodes.py) to send custom functions to the Clipper. This
is used when a button normally assigned to an LED needs to send the Clipper a function code, which may be different
from the channel it normally communicates on."""
		statusByte = MIDI_NOTE_ON_STATUS + FUNCTION_CHANNEL
		functionCode = functionCodeForLed(functionCode, self.ledNumber)
			
		# When sending functions to the fingerboard, the velocity value identifies the LED's offset from 128
		velocityValue = self.ledNumber % 128
		if self._send_midi((statusByte, functionCode, velocityValue), optimized=False):
			self.lastSentFunction = functionCode
