import Live
from _Framework.ClipSlotComponent import ClipSlotComponent
from _Framework.ButtonElement import ButtonElement
from _Framework.SubjectSlot import subject_slot

import diagnostic

from FingerboardButton import FingerboardButton
from fingerboardcodes import *
from colorlookup import *

class LiveZoneClipSlot(ClipSlotComponent):
	"""Represents a clip slot controlled from the Live Zone on the Ztar fingerboard."""
	def __init__(self):
		ClipSlotComponent.__init__(self)
		
		self._lastColor = sharedColorInfo['lookup'][COLOR_LEDOFF]
		self.set_triggered_to_record_value(sharedColorInfo['lookup'][COLOR_RED])
		self.set_recording_value(sharedColorInfo['lookup'][COLOR_RED])
	
	def update(self):
		"""Overrides the default implementation to include blink/brightness function codes in the button update."""
		self._has_fired_slot = False
                button = self._launch_button_value.subject
                if self._allow_updates:
                        if self.is_enabled() and button:
                                valueToSend = self._feedback_value()
                                if self._clip_slot != None and self.has_clip():
                                        valueToSend = sharedColorInfo['lookup'][self._clip_slot.clip.color]
                                if valueToSend and isinstance(valueToSend, int) and valueToSend in range(128):
                                        button.send_value(valueToSend)
                                #else:
                                #        button.turn_off()

                                shouldBlink = self._shouldBlink()
                                shouldBeBright = self._shouldBeBright()
                                if shouldBlink:
                                        button.send_function(FUNC_BLINKON)
                                else:
                                        button.send_function(FUNC_BLINKOFF)
                                if shouldBeBright:
                                        button.send_function(FUNC_BRIGHT)
                                else:
                                        button.send_function(FUNC_DIM)
                else:
                        self._update_requests += 1
        
        def _shouldBlink(self):
                if not self._clip_slot:
                        return False
                if self.has_clip():
                        return self._clip_slot.clip.is_triggered
                else:
                        return self._clip_slot.is_triggered

        def _shouldBeBright(self):
                if not self._clip_slot:
                        return False
                return self.has_clip() and self._clip_slot.clip.is_playing
	
	def set_clip_slot(self, clip_slot):
		"""Overrides the default ClipSlotComponent method to add a color listener."""
                #diagnostic.logMessage('Setting clipslot: ' + str(clip_slot))
                if clip_slot != None and clip_slot.clip != None:
                        self.setCurrentColor(clip_slot.clip.color)
		super(LiveZoneClipSlot, self).set_clip_slot(clip_slot)
	
        @subject_slot('color')
	def _on_clip_color_changed(self):
                #diagnostic.logMessage('Color change!')
		self.setCurrentColor()
		self.update()
	
	def setCurrentColor(self, rgbColor=None):
		"""Sets the current color code of this clip slot to the specified color, or to the color of the current
clip if no color is specified."""
		if rgbColor is None:
			if self._clip_slot and self.has_clip():
				colorcode = sharedColorInfo['lookup'].useColor(self._clip_slot.clip.color)
			else:
				colorcode = sharedColorInfo['lookup'][COLOR_WHITE]
                else:
                        colorcode = sharedColorInfo['lookup'].useColor(rgbColor)
		# Inform the lookup table that we're no longer using the last color, so it knows if it can free that address
		sharedColorInfo['lookup'].releaseColor(self._lastColor)
		self._lastColor = colorcode
		
		self.set_triggered_to_play_value(colorcode)
		self.set_started_value(colorcode)
		self.set_stopped_value(colorcode)

        def dumpState(self, fLog):
                if not self._clip_slot:
                        fLog("No clip slot for this component.")
                        return
                fLog("\tHas clip? " + str(self.has_clip()))
                fLog("\tEnabled? " + str(self._is_enabled))
                fLog("\tAllowing updates? " + str(self._allow_updates))
                fLog("\tPlaying state: " + str(self._clip_slot.playing_status))
                fLog("\tTrigger value (play): " + str(self._triggered_to_play_value))
                fLog("\tTrigger value (rec): " + str(self._triggered_to_record_value))
                fLog("\tStart value: " + str(self._started_value))
                fLog("\tStop value: " + str(self._stopped_value))
                fLog("\tRecording value: " + str(self._recording_value))

