from _Framework.SceneComponent import SceneComponent
from _Framework.SubjectSlot import subject_slot

from LiveZoneClipSlot import LiveZoneClipSlot
from fingerboardcodes import FUNC_BRIGHT, FUNC_BLINKON, FUNC_BLINKOFF

import diagnostic

class LiveZoneScene(SceneComponent):
	clip_slot_component_type = LiveZoneClipSlot

	def __init__(self, num_slots, tracks_to_use_callback):
		super(LiveZoneScene, self).__init__(num_slots, tracks_to_use_callback)
		self.default_value = 0
	
	def set_default_value(self, value):
		if value not in range(128): return None
		self.default_value = value
	
	def set_triggered_value(self, value):
		if value not in range(128): return None
		self._triggered_value = value

        @subject_slot('is_triggered')
	def _on_is_triggered_changed(self):
		"""Responds to the Live event generated by the connected scene when its triggered state changes. Also called
by the update() method when the scene is connected and enabled."""
		if not self._scene: return None
		
		if self.is_enabled() and self._launch_button:
			if self._scene.is_triggered:
				self._launch_button.send_value(self._triggered_value)
				self._launch_button.send_function(FUNC_BLINKON)
				self._launch_button.send_function(FUNC_BRIGHT)
			else:
				self._launch_button.send_value(self.default_value)
				self._launch_button.send_function(FUNC_BLINKOFF)
				self._launch_button.send_function(FUNC_BRIGHT)

        def dumpState(self, fLog):
                if not self._scene:
                        fLog("No scene for this component.")
                        return
                fLog("\tTrigger value: " + str(self._triggered_value))
                fLog("\tEnabled? " + str(self._is_enabled))
                fLog("\tAllowing updates? " + str(self._allow_updates))
